import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

import {
  InstanceNameType,
} from '@overflow/commons/di/type';

export interface InjectableAnnotationAttributes {
  name?: InstanceNameType[];
}

export class InjectableAnnotation implements Annotation {
  public readonly attributes: InjectableAnnotationAttributes;

  public constructor(name: InstanceNameType | InstanceNameType[] | InjectableAnnotationAttributes) {

    if (undefined === name) {
      this.attributes = {
      };
      return;
    }

    if (TypeUtil.isString(name)) {
      this.attributes = {
        name: [<InstanceNameType>name],
      };
    } else {
      this.attributes = <InjectableAnnotationAttributes>name;
    }
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    console.log('Injectable');
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('Injectable');
  }

}

export const Injectable = Decorator.create(InjectableAnnotation);
