import {
  ClassType, PropertyKeyType,
} from '@overflow/commons/core/type';

import {
  Class,
} from '@overflow/commons/core/reflect';

import {
  InstanceNameType,
} from '@overflow/commons/di/type';
import { InjectableAnnotation } from '@overflow/commons/di/decorators';

export class InstanceDefinition {
  private _instanceClass: Class;
  private _injectableAnnotation: InjectableAnnotation;

  public constructor(instanceClassType: ClassType) {
    let clazz = Class.forClassType(instanceClassType);
    if (undefined === clazz) {
      throw new Error(`Class[${instanceClassType.name}] is not injectable type. Add @Injectable annotation.`);
    }

    let injectableAnnotation: InjectableAnnotation = clazz.getOwnAnnotation(InjectableAnnotation);
    if (undefined === injectableAnnotation) {
      throw new Error(`Class is not Injectable type. Add @Injectable annotation to class[${instanceClassType.name}]`);
    }
    this._injectableAnnotation = injectableAnnotation;
  }

  public get instanceClass(): Class {
    return this._instanceClass;
  }

  /**
   * getAttribute
   */
  public get annotation(): InjectableAnnotation {
    return this._injectableAnnotation;
  }

}
